"""
A convenient dot dict class

a = dotdict({"argument": "value"})

print (a.argument)

"""


# noinspection PyPep8Naming
class dotdict(dict):
    """dot.notation access to dictionary attributes"""

    def __getattr__(self, attr):
        return self.get(attr)

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__
